=== modified file 'libunity/Makefile.am'
--- libunity/Makefile.am	2009-12-14 15:00:03 +0000
+++ libunity/Makefile.am	2009-12-16 13:25:32 +0000
@@ -18,6 +18,7 @@
   -H unity.h -d . \
   --library unity \
   --vapidir=$(top_builddir)/vapi/ \
+  --thread \
   --pkg clutter-1.0 \
   --pkg clutk-0.3 \
   --pkg gtk+-2.0 \
@@ -28,6 +29,7 @@
   $(libunity_la_VALASOURCES:.vala=.c)
 
 libunity_la_VALASOURCES = \
+  perf-logger.vala \
   shell.vala
 
 libunity_la_APIFILES = \

=== added file 'libunity/perf-logger.vala'
--- libunity/perf-logger.vala	1970-01-01 00:00:00 +0000
+++ libunity/perf-logger.vala	2009-12-16 13:25:32 +0000
@@ -0,0 +1,115 @@
+/*
+ * Copyright (C) 2009 Canonical Ltd
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 3 as
+ * published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Authored by Gord Allott <gord.allott@canonical.com>
+ *
+ */
+
+namespace Unity
+{
+  
+  public class ProcessInfo
+  {
+    public ProcessInfo (string name)
+    {
+      this.name = name;
+      start = 0;
+      end = 0;
+    }
+    public string name;
+    public double start;
+    public double end;
+  }
+  
+  public TimelineLogger? timeline_singleton;
+  
+  public class TimelineLogger : Object
+  {
+    private Timer global_timer;
+    private Gee.HashMap<string, ProcessInfo> process_map;
+    
+    public static unowned Unity.TimelineLogger get_default ()
+    {
+      if (Unity.timeline_singleton == null)
+      {
+        Unity.timeline_singleton = new Unity.TimelineLogger ();
+      }
+      
+      return Unity.timeline_singleton;
+    }
+    
+    construct
+    {
+      this.process_map = new Gee.HashMap<string, ProcessInfo> ();
+      this.global_timer = new Timer ();      
+      this.global_timer.start ();
+    }
+    
+    public void start_process (string name)
+    {
+      if (name in this.process_map.keys)
+      {
+        warning ("already started process: %s", name);
+        return;
+      }
+      
+      var info = new ProcessInfo (name);
+      this.process_map[name] = info;
+      info.start = this.global_timer.elapsed ();
+    }
+    
+    public void end_process (string name)
+    {
+      double end_time = this.global_timer.elapsed ();
+      
+      if (name in this.process_map.keys)
+      {
+        this.process_map[name].end = end_time;
+      }
+      else 
+      {
+        warning ("process %s not started", name);
+      }
+    }
+    
+    public void write_log (string filename)
+    {
+      debug ("Writing performance log file: %s...", filename);
+      var log_file = File.new_for_path (filename);
+      FileOutputStream file_stream;
+      try 
+      {
+        if (!log_file.query_exists (null)) {
+          file_stream = log_file.create (FileCreateFlags.NONE, null);
+        }
+        else 
+        {
+          file_stream = log_file.replace (null, false, FileCreateFlags.NONE, null);
+        }
+        
+        var output_stream = new DataOutputStream (file_stream);
+        
+        foreach (ProcessInfo info in this.process_map.values)
+        {
+          string outline = "%s, %f, %f\n".printf(info.name, info.start, info.end);
+            output_stream.put_string (outline, null); 
+        }
+      } catch (Error e)
+      {
+        warning (e.message);
+      }
+    }
+  }
+}

=== modified file 'src/Makefile.am'
--- src/Makefile.am	2009-12-14 18:34:48 +0000
+++ src/Makefile.am	2009-12-16 13:25:32 +0000
@@ -29,6 +29,7 @@
 libunity_static_la_VALAFLAGS = \
   -C \
   -H unity-static.h -d . \
+  --thread \
   --library unity-static \
   --vapidir=$(top_srcdir)/vapi/ \
   --vapidir=$(top_srcdir)/libunity/ \
@@ -109,6 +110,7 @@
   --vapidir=./ \
   --vapidir=$(top_srcdir)/libunity/ \
   --vapidir=$(top_srcdir)/vapi/ \
+  --thread \
   --pkg clutk-0.3 \
   --pkg clutter-1.0 \
   --pkg clutter-gtk-0.10 \

=== modified file 'src/main.vala'
--- src/main.vala	2009-12-10 17:39:09 +0000
+++ src/main.vala	2009-12-16 13:25:32 +0000
@@ -22,6 +22,7 @@
 static int  popup_width  = 1024;
 static int  popup_height = 600;
 static bool show_version = false;
+static string? boot_logging_filename = null;
 
 const OptionEntry[] options = {
   {
@@ -70,6 +71,15 @@
     null
   },
   {
+    "enable-boot-logging",
+    'b',
+    OptionFlags.FILENAME,
+    OptionArg.FILENAME,
+    ref boot_logging_filename,
+    "show the boobady bah and foo",
+    null
+  },
+  {
     null
   }
 };
@@ -108,6 +118,7 @@
       }
 
     /* Unique instancing */
+    Unity.TimelineLogger.get_default().start_process ("/Unity");
     app = new Unity.Application ();
     if (app.is_running)
       {
@@ -125,14 +136,27 @@
         
         return response == Unique.Response.OK ? 0 : 1;
       }
+    Unity.TimelineLogger.get_default().end_process ("/Unity");
 
     /* Things seem to be okay, load the main window */
+    Unity.TimelineLogger.get_default().start_process ("/Unity/Window");
     window = new Unity.UnderlayWindow (popup_mode, popup_width, popup_height);
     app.shell = window;
-
     window.show ();
+    Unity.TimelineLogger.get_default().end_process ("/Unity/Window");
+    
+    if (boot_logging_filename != null)
+    {
+      Timeout.add_seconds (3, () => {
+        Unity.TimelineLogger.get_default().write_log ("stats.log");
+        return false;
+      });
+    }
     Gtk.main ();
 
+    
+    
+
     return 0;
   }
 }

=== modified file 'src/quicklauncher/quicklauncher-manager.vala'
--- src/quicklauncher/quicklauncher-manager.vala	2009-12-14 14:14:37 +0000
+++ src/quicklauncher/quicklauncher-manager.vala	2009-12-16 13:25:32 +0000
@@ -49,6 +49,8 @@
       this.container.drag_motion.connect (on_drag_motion);
       this.container.drag_drop.connect (on_drag_drop);
       this.container.drag_data_received.connect (on_drag_data_received);
+      
+      Unity.TimelineLogger.get_default().end_process ("/Unity/Quicklauncher");
     }
     
     private bool on_drag_motion (Ctk.Actor actor, Gdk.DragContext context, 
@@ -166,6 +168,7 @@
 
     private void build_favorites () 
     {
+      Unity.TimelineLogger.get_default().start_process ("/Unity/Quicklauncher/Favorites");
       var favorites = Launcher.Favorites.get_default ();
       
       unowned SList<string> favorite_list = favorites.get_favorites();
@@ -189,6 +192,8 @@
               add_view (view);
             }
         }
+      
+      Unity.TimelineLogger.get_default().end_process ("/Unity/Quicklauncher/Favorites");
     }
 
 

=== modified file 'src/quicklauncher/quicklauncher.vala'
--- src/quicklauncher/quicklauncher.vala	2009-12-07 14:55:22 +0000
+++ src/quicklauncher/quicklauncher.vala	2009-12-16 13:25:32 +0000
@@ -32,6 +32,7 @@
 
     construct
     {
+      Unity.TimelineLogger.get_default().start_process ("/Unity/Quicklauncher");
       this.manager = new Manager ();
       this.add_actor (manager);
     }

=== modified file 'src/window.vala'
--- src/window.vala	2009-12-10 18:50:15 +0000
+++ src/window.vala	2009-12-16 13:25:32 +0000
@@ -99,8 +99,11 @@
       };
 
       Ctk.dnd_init (this.gtk_clutter, target_list);
-            
+      
+      Unity.TimelineLogger.get_default().start_process ("/Unity/Stage/Pre-Paint");
       this.stage = (Clutter.Stage)this.gtk_clutter.get_stage ();
+      this.stage.paint.connect (this.first_stage_paint);
+      
       Clutter.Color stage_bg = Clutter.Color () { 
           red = 0x00,
           green = 0x00,
@@ -135,6 +138,12 @@
       Unity.TooltipManager.get_default().top_level = this;
     }
 
+    private void first_stage_paint ()
+    {
+      this.stage.paint.disconnect (this.first_stage_paint);
+      Unity.TimelineLogger.get_default().end_process ("/Unity/Stage/Pre-Paint");
+    }
+
     private void relayout ()
     {
       int x, y, width, height;

=== added directory 'tools'
=== added file 'tools/makebootchart.py'
--- tools/makebootchart.py	1970-01-01 00:00:00 +0000
+++ tools/makebootchart.py	2009-12-16 13:25:32 +0000
@@ -0,0 +1,247 @@
+#!/usr/bin/env python
+#
+# Copyright (C) 2009 Canonical Ltd
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 3 as
+# published by the Free Software Foundation.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+# Authored by Gordon Allott <gord.allott@canonical.com>
+#
+#
+import os, sys
+import getopt
+import cairo
+import csv
+import math
+import random
+from string import Template
+from socket import gethostname 
+from datetime import datetime 
+import re
+import subprocess
+
+header = Template("""Unity bootchart for $hostname ($date)
+uname: $uname
+CPU: $cpu
+GPU: $gpu
+time: $total_time""")
+
+def sort_by_domain (x, y):
+  domain_x = x["name"].split("/")
+  domain_y = y["name"].split("/")
+  
+  def recurse_domain (a, b, pos, xstart, ystart):
+    
+    if len(a) > len(b):
+      #check to see if the last item of the shortest is the same
+      if a[len(b)-1] == b[-1]:
+        return -1
+      
+      #not in the same tree, 
+      return int(ystart - xstart)
+      
+    if len(b) > len(a):
+      if b[len(a)-1] == a[-1]:
+        return -1
+      
+      return int(ystart - xstart)
+    
+    if pos + 1 > len(a):
+      return -1
+    if pos + 1 > len(b):
+      return +1
+    
+    if (pos+2 > len(a) and pos+2 > len(b)):
+      return int(ystart - xstart)
+    
+    if a[pos] == b[pos]:
+      return recurse_domain(a, b, pos+1, xstart, ystart)
+      
+    if a[pos] < b[pos]:
+      return -1
+    
+    if a[pos] > b[pos]:
+      return +1
+  
+  return recurse_domain (domain_x, domain_y, 0, x['start'], y['start'])
+    
+    
+def gatherinfo (filename):
+  date =  datetime.fromtimestamp(os.path.getmtime(filename))
+  
+  cpufile = open ("/proc/cpuinfo")
+  cpuinfo = cpufile.read (10024)
+  cpure = re.search (r"^model name\s*: (.*$)", cpuinfo, re.MULTILINE)
+  cpu = cpure.group(1)
+  
+  gpu_prog = subprocess.Popen("glxinfo", stdout=subprocess.PIPE)
+  gpu_prog.wait () 
+  gpuinfo = gpu_prog.stdout.read (10024)
+  gpure = re.search (r"^OpenGL renderer string: (.*$)", gpuinfo, re.MULTILINE)
+  gpu = gpure.group (1)
+  
+  return {"hostname":gethostname(),
+          "date": date.strftime("%A, %d. %B %Y %I:%M%p"),
+          "uname": " ".join (os.uname ()),
+          "cpu": cpu,
+          "gpu": gpu,
+          "total_time": "undefined"
+          }
+
+width_multiplier = 1000
+bar_height = 16
+
+def draw_bg_graph (ctx, seconds, height):
+  
+  total_width = seconds * width_multiplier
+  ctx.set_source_rgba (0.0, 0.0, 0.0, 0.25)
+  
+  ctx.move_to (0, 0)
+  ctx.line_to (total_width, 0)
+  ctx.stroke ()
+  
+  per_ten = 0
+  for pos in xrange (0, int(total_width), int (0.01 * width_multiplier)):
+    ctx.set_line_width (1)
+    ctx.set_source_rgba (0.0, 0.0, 0.0, 0.10)
+
+    if (not per_ten):
+      ctx.set_line_width (2)
+      ctx.set_source_rgba (0.0, 0.0, 0.0, 0.25)
+      ctx.move_to (pos-6, -2)
+      ctx.show_text (str (pos / float(width_multiplier)))
+      ctx.stroke ()
+
+    ctx.move_to (pos, 0)
+    ctx.line_to (pos, height)
+    ctx.stroke ()
+    
+    per_ten += 1
+    per_ten %= 10
+
+    
+
+    
+
+def build_graph (data, filename, info):
+  
+  padding_left = 6
+  padding_right = 6
+  padding_top = 6
+  padding_bottom = 6
+  
+  total_size = 0.0
+  for item in data:
+    if item['end'] > total_size:
+      total_size = item['end']
+      
+  width = total_size * width_multiplier + padding_left + padding_right
+  height = len(data) * (bar_height+6) + 60 + padding_left + padding_right
+  surface = cairo.SVGSurface(filename, width, height)
+  
+  ctx = cairo.Context (surface)
+
+  #fill background
+  ctx.set_source_rgb (1, 1, 1)
+  ctx.rectangle (0, 0, width, height)
+  ctx.fill ()
+  
+  #print header
+  info['total_time'] = "%s secs" % total_size
+  sheader = header.substitute(info)
+  
+  ctx.translate (padding_left, padding_right)
+  ctx.set_source_rgb (0, 0, 0)
+  for line in sheader.split("\n"):
+    ctx.translate (0, 12)
+    ctx.show_text (line)
+    ctx.fill ()
+  
+  ctx.translate (0, 12)
+  
+  draw_bg_graph (ctx, total_size, len (data) * bar_height + 16)
+  
+  ctx.set_line_width (1)
+  for item in data:
+    x = item['start'] * width_multiplier
+    x1 = (item['end'] - item['start']) * width_multiplier
+    ctx.translate (x, 0)
+    
+    ctx.set_source_rgba (0.35, 0.65, 0.8, 0.5)
+    ctx.rectangle (0, 0, x1, 16)
+    ctx.fill ()
+    
+    ctx.set_source_rgba (0.35, 0.65, 0.8, 1.0)
+    ctx.rectangle (0, 0, x1, 16)
+    ctx.stroke ()
+    
+    ctx.translate (8, 10)
+    ctx.set_source_rgb (0.0, 0.0, 0.0)
+    ctx.show_text (item['name'])
+    ctx.fill()
+    
+    ctx.translate (-x-8, 6)
+    
+
+  
+  
+def build_data_structure (input):
+  reader = csv.reader(open(input))
+  structure = []
+  print "reading", input
+  for row in reader:
+    name = row[0]
+    start = float(row[1])
+    end = float(row[2])
+    structure.append ({"name": name, "start": start, "end": end})
+    
+  return structure
+    
+
+def usage():
+  print "use --input=filename.log and --output=filename.svg :)" 
+
+def main():
+
+  try:
+      opts, args = getopt.getopt(sys.argv[1:], "h", ["help", "output=", "input="])
+  except getopt.GetoptError, err:
+    # print help information and exit:
+    print str(err) # will print something like "option -a not recognized"
+    usage()
+    sys.exit(2)
+    
+  output = None
+  input = None
+  for o, a in opts:
+    if o in ("-h", "--help"):
+      usage()
+      sys.exit()
+    elif o in ("--output"):
+      output = a
+    elif o in ("--input"):
+      input = a
+    else:
+      assert False, "unhandled option"
+
+  if (not output or not input):
+    usage()
+    sys.exit()
+    
+  data = build_data_structure (input)
+  info = gatherinfo (input)
+  build_graph (data, output, info)
+
+
+  return 0
+
+if __name__ == '__main__': main()

